// IL_BatchEdit.ls 
// --------------------------------------------------------------------
//
// 2.0.4 * 27-SEP-2005: Testet with LW8.x
//
// 2.0.3 * 21-NOV-2002: Update/GUI rewrite by Scott Wheeler
//                      MeshEdit PIs supported now (see Reduce-Polygons)
//
// 2.0.2 * 16-JUN-2002: Updated for LW7.5 
//
// --------------------------------------------------------------------
// purpose:
// - batch edit objects
// - batch convert from/to 3ds/dxf/obj
// - backup/restore object files
// --------------------------------------------------------------------
// Notes:
// - "Surface filter (RegExp) ...."
//   Selects polys with surfaces matching the given regular 
//   expression. Only selected polygons will be modified.
//
//   .* skips polygon selection (all polygons will be modified)
//
//   examples:
//   .*         any string
//   ^Null      begins with "Null"
//   Goal$      ends with "Goal"
//   ^Surf1$    is exactly "Surf1"
//   ^...S      begins with any 3 chars followed by "S"
//   TARGET     contains "TARGET"
//   (Regular expressions are case sensitive)
//   
//   Flatten Layers is not affected by the surface filter and will
//   be applied to all polygons first.
//   
// --------------------------------------------------------------------
// Caution:
// Do not work with your original files. 
// Use copies in a temp directory.
//
// --------------------------------------------------------------------
// http://interialabs.de/lw/lscript
// diver@interialabs.de
//
// Code by Reiner Schug * (c) 1998-2005 
// --------------------------------------------------------------------


@script modeler
@version 2.5
@name IL_BatchEdit2

@define WINTITLE "Batch Object Edit 2.0.4"

var lwos[9999];  // obj filenames
var dirsplit[4];    
var tokens[20];      
f;              // file obj

selx=".*";
selexpr;
cy;

main
{
    var pad[5];
    pad[1] = "0";
    pad[2] = "00";
    pad[3] = "000";
    pad[4] = "0000";
    pad[5] = "";

    surf = "Default";
    newsurf = surf;
    wc=1;           // local/world flag
    sdmaxangle=179; // for subdiv

    expertmode=recall("expertmode", false);
    if(!expertmode)
        if(!MessageBox(WINTITLE, "Warning! All Objects will be closed now.") )
            return;

    closeall();

    contentdir= getdir("Content");

    // panel
    dy  = 22; 
    cx  = 15;
    cxoff = cx+80;
    cx2 = 200;
    cy  = 23;
    

    reqbegin(WINTITLE);
    reqsize(310,665);

    c100 = ctltab("Modify", "Options", "Export"); ctlposition(c100, 4,4);
 
    // TAB 1
    cy=24;
    sep = ctlsep(-1,-1); ctlposition(sep, 0, cy); cy+=8;

    c99 = ctlchoice("Edit objects in a", 1, @"Directory","Scene"@); ctlposition(c99, (cxoff - 80), cy,285); cy+=dy;
    c1 = ctlfilename("Scene ContentDir", contentdir, 30, true); ctlposition(c1, (cxoff - 89), cy,271); cy+=dy;
    cy+=4; sep7 = ctlsep(-1,-1); ctlposition(sep7, 0, cy); cy+=8;


    c2 = ctlcheckbox("Change Surface", false); ctlposition(c2, cxoff, cy); cy+=dy;
    c3 = ctlsurface("Surface:", surf); ctlposition(c3, (cxoff - 46), cy,253); cy+=dy;
    c93 = ctlcheckbox("Numbered surfaces?",false); ctlposition(c93, cxoff, cy); cy+=dy;
    cy+=4; sep1 = ctlsep(-1,-1); ctlposition(sep1, 0, cy); cy+=8;

    c5 = ctlcheckbox("Scale XYZ",false); ctlposition(c5, cxoff, cy); cy+=dy;
    c6 = ctlvector("", <1,1,1>); ctlposition(c6, (cxoff - 6), cy); cy+=dy;
    cy+=4; sep2 = ctlsep(-1,-1); ctlposition(sep2, 0, cy); cy+=8;

    c7 = ctlcheckbox("Rotate XYZ (PHB)",false); ctlposition(c7, cxoff, cy); cy+=dy;
    c8 = ctlvector("", <0,0,0>); ctlposition(c8, (cxoff - 6), cy); cy+=dy;
    c81 = ctlchoice("", 2, @"Object","<0,0,0>"@); ctlposition(c81, (cxoff - 6), cy,212); cy+=dy;
    cy+=4; sep3 = ctlsep(-1,-1); ctlposition(sep3, 0, cy); cy+=8;

    c17 = ctlcheckbox("Center Data", false); ctlposition(c17, cxoff, cy); cy+=dy;
    cc = ctlchoice("Center On", 4, @"X","Y","Z","ALL"@); ctlposition(cc, (cxoff - 54), cy,212); cy+=dy;
    cy+=4; ccsep = ctlsep(-1,-1); ctlposition(ccsep, 0, cy); cy+=8;

    c11 = ctlcheckbox("Subdivide",false); ctlposition(c11, cxoff, cy); cy+=dy;
    c13 = ctlinteger("Max. Angle", sdmaxangle); ctlposition(c13, (cxoff - 59), cy,264); cy+=dy;
    c12 = ctlchoice("", 1, @"flat","smooth","metaform"@,false); ctlposition(c12, (cxoff - 6), cy,212); cy+=dy;
    cy+=4; sep4 = ctlsep(-1,-1); ctlposition(sep4, 0, cy); cy+=8;

    c9  = ctlcheckbox("Merge Points", false); ctlposition(c9, cxoff, cy); cy+=dy;
    c10 = ctlnumber("Min Dist (0=auto)", 0); ctlposition(c10, (cxoff - 86), cy,291); cy+=dy;
    cy+=4; sep5 = ctlsep(-1,-1); ctlposition(sep5, 0, cy); cy+=8;

    //cy+=dy;
    c32 = ctlcheckbox("Flatten Layers", false); ctlposition(c32, cxoff, cy); cy2=cy; cy+=dy;
    c34 = ctlcheckbox("Rest on Ground", false); ctlposition(c34, cxoff, cy); cy+=dy;
    c4  = ctlcheckbox("Triple", false); ctlposition(c4, cxoff, cy); cy+=dy;
    c14 = ctlcheckbox("Flip Polys", false); ctlposition(c14, cxoff, cy); cy+=dy;

    c15 = ctlcheckbox("Unify Polys", false); ctlposition(c15, cx2, cy2); cy2+=dy;
    rp  = ctlcheckbox("Reduce Polys", false); ctlposition(rp, cx2, cy2); cy2+=dy;

    c22 = ctlcheckbox("Toggle SubDs", false); ctlposition(c22, cx2, cy2); cy2+=dy;
    cy+=4; sep6 = ctlsep(-1,-1); ctlposition(sep6, 0, cy); cy+=8;

  //  cy+=dy;
    c35 = ctlstring("Surface filter", selx, 200); ctlposition(c35, (cxoff - 64), cy,269); cy+=dy;
    c36 = ctlcheckbox("exclude?",false); ctlposition(c36, cxoff, cy); cy+=dy+11;
    ctlpage(1, sep,sep1,sep2,sep3,sep4,sep5,sep6,sep7,ccsep,cc,c99,c1,c2,c3,c93,c5,c6,c7,c8,c81,c11,c12,c13,c4,c14,c15,rp,c17,c22,c9,c10,c32,c34,c35,c36);

    // TAB 2
    cy=24;
    t2sep = ctlsep(-1,-1); ctlposition(t2sep, 0, cy); cy+=8;

    c16 = ctlcheckbox("Create backup files (*.lwo.bak)", false); ctlposition(c16, cx, cy); cy+=dy;
    c33 = ctlbutton("Restore backup files now", 170 , "bt_restorebak"); ctlposition(c33, cx, cy); cy+=dy*2;
    c31 = ctlcheckbox("Expert Mode (No warnings)", expertmode); ctlposition(c31, cx, cy); cy+=dy;
    c30 = ctlcheckbox("Debug messages", false); ctlposition(c30, cx, cy); cy+=dy;
    ctlpage(2, t2sep,c16,c30,c31,c33);

    // TAB 3
    cy=24;
    t3sep = ctlsep(-1,-1); ctlposition(t3sep, 0, cy); cy+=8;
    c41 = ctlcheckbox("3DS (triple first!)", false); ctlposition(c41, cx, cy); cy+=dy;
    c42 = ctlcheckbox("DXF", false); ctlposition(c42, cx, cy); cy+=dy;
    c43 = ctlcheckbox("OBJ", false); ctlposition(c43, cx, cy); cy+=dy*3;
    c49 = ctltext("Modify settings will have no effect for export.","Modify first, then export."); ctlposition(c49, cx, cy);
    ctlpage(3, t3sep,c41,c42,c43,c49);

    reqresize("resize");

    ctlactive(c2,"Surf",c3);
    ctlactive(c2,"Surf",c93);

    ctlactive(c5,"Scale",c6);
    ctlactive(c7,"Rotate",c8);
    ctlactive(c7,"Rotate",c81);
    ctlactive(c17,"Center",cc);

    ctlactive(c11,"SubD",c13);
    ctlactive(c11,"SubD",c12);
    ctlactive(c9,"SubD",c10);


    ctlactive(c99,"Contscene",c1);

    if(reqpost())
    {
        contentdir =    getvalue(c1);

        ed_surf =       getvalue(c2);
        newsurf =       getvalue(c3);
        surfprogress =  getvalue(c93);

        ed_scale =      getvalue(c5);
        scaling =       getvalue(c6);
        ed_rotate =     getvalue(c7);
        rotation =      getvalue(c8);
        wc       =      getvalue(c81);

        ed_subdiv =     getvalue(c11);
        sdmode =        getvalue(c12);
        sdmaxangle =    getvalue(c13);

        applymode =     getvalue(c99);

        ed_flatten =    getvalue(c32);
        ed_restong =    getvalue(c34);
        ed_triple =     getvalue(c4);
        ed_flip =       getvalue(c14);
        ed_unify =      getvalue(c15);
        ed_reducepoly = getvalue(rp);
        ed_center =     getvalue(c17);
        centertype    = getvalue(cc);
        ed_toglpatch =  getvalue(c22);

        ed_merge =      getvalue(c9);
        mergedist =     getvalue(c10);

        makebak =       getvalue(c16);
        debugmode =     getvalue(c30);
        expertmode =     getvalue(c31);
        store("expertmode", expertmode);

        ex_3ds = getvalue(c41);
        ex_dxf = getvalue(c42);
        ex_obj = getvalue(c43);

        selx = getvalue(c35);
        selx_exclude = getvalue(c36);
    }
    else
        return;
    reqend();

    if(selx==".*" && selx_exclude) error("RegExp: Exclude all???");
    selexpr=regexp(selx);
    if(selexpr==nil) 
        error("Regular expression didn't compile.");

    /// 
    if(ex_3ds+ex_dxf+ex_obj>0)
    {
        ex_sel=true;
    }
    else
    {
        if (ed_surf+ed_triple+ed_scale+ed_rotate+ed_merge+ed_subdiv+ed_flip+
            ed_unify+makebak+ed_center+ed_toglpatch+ed_flatten+ed_restong+ed_reducepoly == 0)
        {
            info("Nothing to do."); return;
        }
    }
    ///////////////////////////////////////////////////////
    // first check how many obj to process

    if (applymode == 2)
    {
        // scan scene for disk objects

        sc = getfile("Load Scene","*.lws", "", true);
        if(sc != nil)
        {
            f = File(sc,"r");
            i=1;
            while(f.eof() == false)
            {
                tokens = f.parse(" ");
                if((f.eof() == false) && (size(tokens) > 2) 
                    && (tokens[1] == "LoadObjectLayer")
                    && (tokens[2] == "1")
                    )  
                {
                    objfile = split(tokens[3]);
                    if(objfile[1]!=nil)
                        lwos[i]=tokens[3];
                    else    
                        lwos[i]=contentdir+"\\"+tokens[3];

                    if (!fileexists(lwos[i]))
                    {
                        f.close();
                        error("aborted! file not found: " +lwos[i]);
                        info("No files changed.");
                        return;
                    }
                    i++;
                }
            }
            f.close();
            i--;
            prestats="Found "+string(i)+" files to process.";
        }
        else
        {
            return;     // filereq canceled
        }
    }
    else
    {
        // scan directory for objects

        contentdir=getdir("Objects");  //!!

        dir=getfile("Select an object in the directory", "*.*", contentdir, true);
        if (dir == nil)
            return;

        dirsplit=split(dir);    
        if(dirsplit[1]!=nil)    // path includes driveletter ?
        {
            lwos=matchfiles(dirsplit[1]+dirsplit[2],"*"+dirsplit[4]);
            i=1;
            while (i<=size(lwos))
            {
                lwos[i]=dirsplit[1]+dirsplit[2]+lwos[i];
                i++;
            }
        }
        else
        {
            lwos=matchfiles(dirsplit[2],"*"+dirsplit[4]);
            i=1;
            while (i<=size(lwos))
            {
                lwos[i]=dirsplit[2]+lwos[i];
                i++;
            }
        }
        i--;

        prestats="Found "+string(i)+" "+dirsplit[4]+" files to process";
        sc=dirsplit[2];
    }   
    numfiles=i;

    ///
    if(!expertmode)
    {
        reqbegin("!WARNING!");
        ctltext("",prestats);
        ctltext("","in "+sc);
        ctltext("","");
        ctltext("", "Existing files will be overwritten! Are you sure?");
        if(!reqpost())
            return;
        reqend();
    }

    i=1;
    lwos.sortA();
    dig=size(string(size(lwos)));
    selmode(GLOBAL);
    while (i<=numfiles)
    {
        if (fileexists(lwos[i]))
            {
 //               

                load(lwos[i]);
               
                if (ed_reducepoly == 1)
                {
                  // reduce.p
                  meshedit("Reduce-Polygons");
                }
        
                if(!ex_sel)
                {
                    npoints=pointcount();
                    head=strright("..."+lwos[i],25);
                    moninit(10,head);

                    if(ed_flatten == 1)
                    {
                        lyrsetfg(lyrdata());
                        lyrs=lyrfg();
                        if(size(lyrs>1))
                        {
                            cut();
                            lyrsetfg(1);
                            paste();
                        }
                    }
                    lyrsetfg(lyrdata());

                    selmode(USER);

                    // surf select
                    scount=0;
                    if(selx != ".*")
                    {
                        selpolygon(CLEAR,NVGT,0);
                        surflist=Surface(Mesh(0));
                        if(debugmode) info("surfaces: ", surflist.size());
                        
                        for(si=1; si<=size(surflist); si++)
                        {
                            
                            if( ((surflist[si]==selexpr) ^ selx_exclude) )
                            {
                                if(debugmode) info("TRUE: ",surflist[si]);
                                selpolygon(SET, SURFACE, surflist[si]);
                                scount++;
                            }
                        }
                    }

                    if( selx == ".*" || (selx != ".*" && scount > 0) )
                    {
                        if(debugmode) info("Modify.");
                        
                        if(monstep()) return;

                        if(ed_restong == 1)
                        {
                            bbox=boundingbox();
                            move( <0, bbox[1].y * -1, 0> );
                        }


                        if (ed_unify == 1)
                            unifypols();


                        if (ed_triple == 1)
                            triple();

                        if (ed_subdiv == 1) 
                        {
                            if (ed_triple == 1)
                            {}
                            else
                            {
                                selpolygon(SET,NVGT,4);
                                needtriple=polycount();
                                selpolygon(CLEAR,NVGT,4);
                                if (needtriple!=nil)
                                {
                                    if (needtriple[6]>0) // triple >4 point polys 
                                    {
                                        selmode(USER);
                                        selpolygon(SET,NVGT,4);
                                        triple();
                                        selpolygon(CLEAR,NVGT,0);
                                    }
                                }
                            }

                            // surf re-select
                            if(selx != ".*")
                            {
                                selpolygon(CLEAR,NVGT,0);
                                surflist=Surface(Mesh(0));
                                for(si=1; si<=size(surflist); si++)
                                {
                                    if( ((surflist[si]==selexpr) ^ selx_exclude) )
                                        selpolygon(SET, SURFACE, surflist[si]);
                                }
                            }

                            if(sdmode>1)
                                subdivide(sdmode);
                            else
                                subdivide(sdmode, sdmaxangle);
                        }

                        if (ed_surf == 1)
                        {
                            //selpolygon(SET,NVGT,0);
                            if(surfprogress == 1)
                            {
                                z=getpad(i,dig);
                                copysurface(newsurf,newsurf+pad[z]+string(i));
                                changesurface(newsurf+pad[z]+string(i)); 
                            }
                            else
                                changesurface(newsurf); 
                        }

                        //selmode(GLOBAL);

                        if (ed_center == 1)
                        {
                          pivc = center(boundingbox());
                          switch(centertype)
                          {
                           case 1:
                            move( <pivc.x*-1,0,0> );
                           break;
 
                           case 2:
                            move( <0,pivc.y*-1,0> );
                           break;

                           case 3:
                            move( <0,0,pivc.z*-1> );
                           break;

                           case 4:
                            move( -center(boundingbox()) );
                           break;
                          } // End Center Switch
                        } // End Center if

                        if (monstep()) return;

                        if (wc==2)
                            piv = <0,0,0>;
                        else
                            piv = center(boundingbox());

                        if (ed_scale == 1)
                            scale(scaling,piv);

                        if (ed_rotate == 1)
                        {
                            if(abs(rotation.z)>0)
                                rotate(rotation.z, 3, piv);
                            if(abs(rotation.x)>0)
                                rotate(rotation.x, 1, piv);
                            if(abs(rotation.y)>0)
                                rotate(rotation.y, 2, piv);
                        }

                        if (monstep()) return;

                        if (ed_merge == 1)
                            mergepoints(mergedist);
                         
                        if (monstep()) return;
            
                        if (ed_toglpatch == 1)
                            togglepatches();

                        if (monstep()) return;
            
                        if (ed_flip == 1)
                            flip();
            
                        if (makebak == 1)
                        {
                            filedelete(lwos[i]+".bak");
                            cmd=getenv("ComSpec")+" /c copy "+lwos[i]+" "+lwos[i]+".bak";
                            spawn(cmd);
                        }
            
                        h=split(lwos[i]);
                        if (h[4] == ".lwo")
                        {
                            save(lwos[i]);
                        }
                        else
                        {
                            if (h[1]!=nil)
                                save(h[1]+h[2]+h[3]+".lwo");
                            else
                                save(h[2]+h[3]+".lwo");
                        }
                    }
                    monend(); //!!!!
                }
                else
                {               
                    //!!!!
                    // careful: crash if monitor with other cmdseq plug is used here!

                    h=split(lwos[i]);
                    if (h[1]!=nil)
                        pre=h[1]+h[2]+h[3];
                    else
                        pre=h[2]+h[3];


                    // add conv3d.p!
                    if(ex_3ds)
                        cmdseq("Export3DS", pre+".3ds");
        
                    if(ex_dxf)
                        cmdseq("ExportDXF", pre+".dxf");

                    if(ex_obj)
                        cmdseq("ExportOBJ", pre+".obj");
                }
            
                close();

            }
            else
            {
                info(lwos[i]+" not found.");
            }
            i++;
    }
    i--;
    if(!expertmode && !ex_sel) 
        info(string(i)+" objects changed in "+sc);
}

resize: w,h {}

Surf: value
{
 return(value);
}

Scale: value
{
 return(value);
}

Rotate: value
{
 return(value);
}

SubD: value
{
 return(value);
}

Merge: value
{
 return(value);
}

Contscene: value
{
 return(value = value - 1);
}

Center: value
{
 return(value);
}


getpad: i, il
{
    if(i < 10) 
        it = il - 1;
    if((i >= 10) & (i < 100))
        it = il - 2;
    if((i >= 100) & (i < 1000))
        it = il - 3;
    if((i >= 1000) & (i < 10000))
        it = il - 4;
    if(i >= 10000)
        it = il - 5;
    if it <= 0  then 
        it = 5; 
    return it;
}

MessageBox: title, msg
{
    result=true;
    reqbegin(title);
    reqsize(200,80);
    ctltext(msg, "");
    if(!reqpost())
        result=false;
    reqend();
    return result;
}

bt_restorebak
{
    contentdir=getdir("Objects");

    dir=getfile("Select an object in the directory", "*.bak", contentdir, true);
    if (dir == nil)
        return;
    
    dirsplit=split(dir);

    baks=matchfiles(dirsplit[1]+dirsplit[2],"*.bak");
    lwos=matchfiles(dirsplit[1]+dirsplit[2],"*.lwo");
    if(size(baks) != size(lwos))
    {
        warn("Number of *.bak must match number of *.lwo");
        reqabort();
        return;
    }

    chdir(dirsplit[1]+dirsplit[2]);
    cmd=getenv("ComSpec")+" /c del *.lwo";
    spawn(cmd);
    cmd=getenv("ComSpec")+" /c ren *.bak *.lwo";
    spawn(cmd);
    reqabort();
}

